//---------------------------------
// 2019/3/15

#include <Arduino.h>

#ifndef BME280_H
#define BME280_H

#define BME280_S32_t  int32_t
#define BME280_S64_t  int64_t
#define BME280_U32_t  uint32_t


class BME280{
public:
  // Calibration Data
  static const uint8_t calLength = 38;
  static const uint8_t calLengthTP = 24;

  union Calibration{
    uint8_t byte[calLength];
    struct{
      uint16_t dig_T1;
      int16_t dig_T2;
      int16_t dig_T3;
      uint16_t dig_P1;
      int16_t dig_P2;
      int16_t dig_P3;
      int16_t dig_P4;
      int16_t dig_P5;
      int16_t dig_P6;
      int16_t dig_P7;
      int16_t dig_P8;
      int16_t dig_P9;
      uint8_t dig_H1;
      uint8_t dummy;
      int16_t dig_H2;
      uint8_t dig_H3;
      uint8_t dummy2;
      int16_t dig_H4;
      int16_t dig_H5;
      int8_t dig_H6;
    };
  };

  union Calibration cal;

  // Device I2C Address
  uint8_t addrI2C;

  // Temperature for calculation
  BME280_S32_t t_fine;

  // Store ADC Register Data
  uint32_t adcT;
  uint32_t adcP;
  uint32_t adcH;

  // Compensate Data
  int32_t compT;
  uint32_t compP;
  uint32_t compH;

  // Measure Results
  float T;  // Degree C
  float P;  // hPa
  float H;  // %


  BME280();
  bool begin(uint8_t addrI2C);

  uint8_t read(uint8_t addr);
  void read(uint8_t addr, uint8_t*data, uint8_t length);
  void write(uint8_t addr, uint8_t data);
  void write(uint8_t addr, uint8_t*data, uint8_t length);

  bool readID();
  uint8_t readStatus();
  void readCal();
  void readADC();

  void forced(bool pooling);

  void printCal();
  void printRegister();
  void printMeas();

  BME280_S32_t compensate_T_int32(BME280_S32_t adc_T);
  BME280_U32_t compensate_P_int64(BME280_S32_t adc_P);
  BME280_U32_t compensate_H_int32(BME280_S32_t adc_H);
};

#endif
